@@ -3,7 +3,7 @@ from abc import ABCMeta, abstractmethod
 from collections import UserDict
 from datetime import datetime, timedelta
 from logging import getLogger
-from typing import Iterable, Optional, Union
+from typing import Callable, Iterable, Optional, Union
 from urllib.parse import parse_qsl, urlparse, urlunparse
 
 from aiohttp import ClientRequest, ClientResponse
@@ -15,41 +15,66 @@ ResponseOrKey = Union[CachedResponse, None, bytes, str]
 logger = getLogger(__name__)
 
 
-class CacheController:
-    """Class to manage higher-level cache operations.
-    Handles cache expiration, and generating cache keys, and managing redirect history.
+class CacheBackend:
+    """Base class for cache backends. This manages higher-level cache operations,
+    including cache expiration, generating cache keys, and managing redirect history.
 
-    Basic storage operations are handled by :py:class:`.BaseCache`.
-    To extend this with your own custom backend, implement a subclass of :py:class:`.BaseCache`
-    to use as :py:attr:`CacheController.responses` and :py:attr:`CacheController.response_aliases`.
+    If instantiated directly, ``CacheBackend`` will use a non-persistent in-memory cache.
+
+    Lower-level storage operations are handled by :py:class:`.BaseCache`.
+    To extend this with your own custom backend, implement one or more subclasses of
+    :py:class:`.BaseCache` to use as :py:attr:`CacheBackend.responses` and
+    :py:attr:`CacheBackend.response_aliases`.
+
+    Args:
+        cache_name: Cache prefix or namespace, depending on backend; see notes below
+        expire_after: Number of hours after which a cache entry will expire; se ``None`` to
+            never expire
+        allowed_codes: Limit caching only for response with this codes
+        allowed_methods: Cache only requests of this methods
+        include_headers: Make request headers part of the cache key
+        ignored_params: List of request parameters to be excluded from the cache key.
+        filter_fn: function that takes a :py:class:`aiohttp.ClientResponse` object and
+            returns a boolean indicating whether or not that response should be cached. Will be
+            applied to both new and previously cached responses
+
+    The ``cache_name`` parameter will be used as follows depending on the backend:
+
+        * ``sqlite``: Cache filename prefix, e.g ``my_cache.sqlite``
+        * ``mongodb``: Database name
+        * ``redis``: Namespace, meaning all keys will be prefixed with ``'cache_name:'``
+
+    Note on cache key parameters: Set ``include_get_headers=True`` if you want responses to be
+    cached under different keys if they only differ by headers. You may also provide
+    ``ignored_parameters`` to ignore specific request params. This is useful, for example, when
+    requesting the same resource with different credentials or access tokens.
     """
 
     def __init__(
         self,
-        cache_name,
-        expire_after=None,
-        filter_fn=None,
-        allowable_codes=None,
-        allowable_methods=None,
-        include_get_headers=False,
-        ignored_parameters=None,
-        **kwargs,
+        cache_name: str = 'http-cache',
+        expire_after: Union[int, timedelta] = None,
+        allowed_codes: tuple = (200,),
+        allowed_methods: tuple = ('GET', 'HEAD'),
+        include_headers: bool = False,
+        ignored_params: Iterable = None,
+        filter_fn: Callable = lambda r: True,
     ):
         self.name = cache_name
         if expire_after is not None and not isinstance(expire_after, timedelta):
             expire_after = timedelta(seconds=expire_after)
         self.expire_after = expire_after
-        self.allowable_codes = allowable_codes
-        self.allowable_methods = allowable_methods
+        self.allowed_codes = allowed_codes
+        self.allowed_methods = allowed_methods
         self.filter_fn = filter_fn
         self.disabled = False
 
         # Allows multiple redirects or other aliased URLs to point to the same cached response
-        self.redirects = DictCache()
-        self.responses = DictCache()
+        self.redirects = DictCache()  # type: BaseCache
+        self.responses = DictCache()  # type: BaseCache
 
-        self._include_get_headers = include_get_headers
-        self._ignored_parameters = set(ignored_parameters or [])
+        self.include_headers = include_headers
+        self.ignored_params = set(ignored_params or [])
 
     def is_cacheable(self, response: Union[ClientResponse, CachedResponse, None]) -> bool:
         """Perform all checks needed to determine if the given response should be cached"""
@@ -58,9 +83,9 @@ class CacheController:
         return all(
             [
                 not self.disabled,
+                response.status in self.allowed_codes,
+                response.method in self.allowed_methods,
                 not self.is_expired(response),
-                response.status in self.allowable_codes,
-                response.method in self.allowable_methods,
                 self.filter_fn(response),
             ]
         )
@@ -82,18 +107,17 @@ class CacheController:
         # Attempt to fetch response from the cache
         try:
             if not await self.responses.contains(key):
-                key = await self.redirects.read(key)
+                key = str(await self.redirects.read(key))
             response = await self.responses.read(key)
         except (KeyError, TypeError):
             return None
+        if not isinstance(response, CachedResponse):
+            return None
 
         # If the item is expired or filtered out, delete it from the cache
         if not self.is_cacheable(response):
             await self.delete(key)
-            try:
-                response.is_expired = True
-            except AttributeError:
-                pass
+            response.is_expired = True
 
         return response
 
@@ -107,8 +131,8 @@ class CacheController:
         if not self.is_cacheable(response):
             return
 
-        response = await CachedResponse.from_client_response(response)
-        await self.responses.write(key, response)
+        cached_response = await CachedResponse.from_client_response(response)
+        await self.responses.write(key, cached_response)
 
         # Alias any redirect requests to the same cache key
         for r in response.history:
@@ -128,7 +152,7 @@ class CacheController:
             for r in response.history:
                 await self.redirects.delete(self.create_key(r.method, r.url))
 
-        redirect_key = await self.redirects.pop(key)
+        redirect_key = str(await self.redirects.pop(key))
         await delete_history(await self.responses.pop(key))
         await delete_history(await self.responses.pop(redirect_key))
 
@@ -169,7 +193,7 @@ class CacheController:
         **kwargs,
     ) -> str:
         """Create a unique cache key based on request details"""
-        if self._ignored_parameters:
+        if self.ignored_params:
             url, params, body = self._remove_ignored_parameters(url, params, data)
 
         key = hashlib.sha256()
@@ -178,7 +202,11 @@ class CacheController:
         key.update(_encode_dict(params))
         key.update(_encode_dict(data))
 
-        if self._include_get_headers and headers and headers != ClientRequest.DEFAULT_HEADERS:
+        if (
+            self.include_headers
+            and headers is not None
+            and headers != ClientRequest.DEFAULT_HEADERS
+        ):
             for name, value in sorted(headers.items()):
                 key.update(name.encode())
                 key.update(value.encode())
@@ -186,7 +214,7 @@ class CacheController:
 
     def _remove_ignored_parameters(self, url, params, data):
         def filter_ignored_params(d):
-            return {k: v for k, v in d.items() if k not in self._ignored_parameters}
+            return {k: v for k, v in d.items() if k not in self.ignored_params}
 
         # Strip off any request params manually added to URL and add to `params`
         u = urlparse(url)
@@ -201,10 +229,9 @@ class CacheController:
 
 
 # TODO: Support yarl.URL like aiohttp does?
-# TODO: Is there an existing ABC for async collections? Can't seem to find any.
 class BaseCache(metaclass=ABCMeta):
-    """A wrapper for the actual storage operations. This is separate from
-    :py:class:`.CacheController` to simplify writing to multiple tables/prefixes.
+    """A wrapper for lower-level cache storage operations. This is separate from
+    :py:class:`.CacheBackend` to allow a single backend to contain multiple cache objects.
 
     This is no longer using a dict-like interface due to lack of python syntax support for async
     dict operations.
@@ -242,14 +269,17 @@ class BaseCache(metaclass=ABCMeta):
     async def write(self, key: str, item: ResponseOrKey):
         """Write an item to the cache"""
 
-    async def pop(self, key: str) -> Optional[ResponseOrKey]:
+    async def pop(self, key: str, default=None) -> Optional[ResponseOrKey]:
         """Delete an item from the cache, and return the deleted item"""
-        item = await self.read(key)
-        await self.delete(key)
-        return item
+        try:
+            item = await self.read(key)
+            await self.delete(key)
+            return item
+        except KeyError:
+            return default
 
 
-class DictCache(UserDict, BaseCache):
+class DictCache(BaseCache, UserDict):
     """Simple in-memory storage that wraps a dict with the :py:class:`.BaseStorage` interface"""
 
     async def delete(self, key: str):
